# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/ray/protobuf/core_worker.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as src_dot_ray_dot_protobuf_dot_common__pb2
from . import pubsub_pb2 as src_dot_ray_dot_protobuf_dot_pubsub__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='src/ray/protobuf/core_worker.proto',
  package='ray.rpc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"src/ray/protobuf/core_worker.proto\x12\x07ray.rpc\x1a\x1dsrc/ray/protobuf/common.proto\x1a\x1dsrc/ray/protobuf/pubsub.proto\"0\n\x0f\x41\x63tiveObjectIDs\x12\x1d\n\nobject_ids\x18\x01 \x03(\x0cR\tobjectIds\"\xc3\x03\n\x0b\x41\x63torHandle\x12\x19\n\x08\x61\x63tor_id\x18\x01 \x01(\x0cR\x07\x61\x63torId\x12\x19\n\x08owner_id\x18\x02 \x01(\x0cR\x07ownerId\x12\x35\n\rowner_address\x18\x03 \x01(\x0b\x32\x10.ray.rpc.AddressR\x0cownerAddress\x12&\n\x0f\x63reation_job_id\x18\x04 \x01(\x0cR\rcreationJobId\x12\x38\n\x0e\x61\x63tor_language\x18\x05 \x01(\x0e\x32\x11.ray.rpc.LanguageR\ractorLanguage\x12q\n\'actor_creation_task_function_descriptor\x18\x06 \x01(\x0b\x32\x1b.ray.rpc.FunctionDescriptorR#actorCreationTaskFunctionDescriptor\x12!\n\x0c\x61\x63tor_cursor\x18\x07 \x01(\x0cR\x0b\x61\x63torCursor\x12%\n\x0e\x65xtension_data\x18\x08 \x01(\x0cR\rextensionData\x12(\n\x10max_task_retries\x18\t \x01(\x03R\x0emaxTaskRetries\"\xba\x01\n\x0cReturnObject\x12\x1b\n\tobject_id\x18\x01 \x01(\x0cR\x08objectId\x12\x1b\n\tin_plasma\x18\x02 \x01(\x08R\x08inPlasma\x12\x12\n\x04\x64\x61ta\x18\x03 \x01(\x0cR\x04\x64\x61ta\x12\x1a\n\x08metadata\x18\x04 \x01(\x0cR\x08metadata\x12,\n\x12nested_inlined_ids\x18\x05 \x03(\x0cR\x10nestedInlinedIds\x12\x12\n\x04size\x18\x06 \x01(\x03R\x04size\"D\n\x11StealTasksRequest\x12/\n\nthief_addr\x18\x01 \x01(\x0b\x32\x10.ray.rpc.AddressR\tthiefAddr\";\n\x0fStealTasksReply\x12(\n\x10stolen_tasks_ids\x18\x02 \x03(\x0cR\x0estolenTasksIds\"\x93\x02\n\x0fPushTaskRequest\x12,\n\x12intended_worker_id\x18\x01 \x01(\x0cR\x10intendedWorkerId\x12.\n\ttask_spec\x18\x02 \x01(\x0b\x32\x11.ray.rpc.TaskSpecR\x08taskSpec\x12\'\n\x0fsequence_number\x18\x03 \x01(\x03R\x0esequenceNumber\x12\x33\n\x16\x63lient_processed_up_to\x18\x04 \x01(\x03R\x13\x63lientProcessedUpTo\x12\x44\n\x10resource_mapping\x18\x05 \x03(\x0b\x32\x19.ray.rpc.ResourceMapEntryR\x0fresourceMapping\"\xd9\x01\n\rPushTaskReply\x12<\n\x0ereturn_objects\x18\x01 \x03(\x0b\x32\x15.ray.rpc.ReturnObjectR\rreturnObjects\x12\x1f\n\x0btask_stolen\x18\x02 \x01(\x08R\ntaskStolen\x12%\n\x0eworker_exiting\x18\x03 \x01(\x08R\rworkerExiting\x12\x42\n\rborrowed_refs\x18\x04 \x03(\x0b\x32\x1d.ray.rpc.ObjectReferenceCountR\x0c\x62orrowedRefs\"g\n%DirectActorCallArgWaitCompleteRequest\x12,\n\x12intended_worker_id\x18\x01 \x01(\x0cR\x10intendedWorkerId\x12\x10\n\x03tag\x18\x02 \x01(\x03R\x03tag\"%\n#DirectActorCallArgWaitCompleteReply\"]\n\x16GetObjectStatusRequest\x12&\n\x0fowner_worker_id\x18\x01 \x01(\x0cR\rownerWorkerId\x12\x1b\n\tobject_id\x18\x02 \x01(\x0cR\x08objectId\"i\n\tRayObject\x12\x12\n\x04\x64\x61ta\x18\x01 \x01(\x0cR\x04\x64\x61ta\x12\x1a\n\x08metadata\x18\x02 \x01(\x0cR\x08metadata\x12,\n\x12nested_inlined_ids\x18\x03 \x03(\x0cR\x10nestedInlinedIds\"\xfc\x01\n\x14GetObjectStatusReply\x12\x42\n\x06status\x18\x01 \x01(\x0e\x32*.ray.rpc.GetObjectStatusReply.ObjectStatusR\x06status\x12*\n\x06object\x18\x02 \x01(\x0b\x32\x12.ray.rpc.RayObjectR\x06object\x12\x19\n\x08node_ids\x18\x03 \x03(\x0cR\x07nodeIds\x12\x1f\n\x0bobject_size\x18\x04 \x01(\x04R\nobjectSize\"8\n\x0cObjectStatus\x12\x0b\n\x07\x43REATED\x10\x00\x12\x10\n\x0cOUT_OF_SCOPE\x10\x01\x12\t\n\x05\x46REED\x10\x02\"h\n\x1dWaitForActorOutOfScopeRequest\x12,\n\x12intended_worker_id\x18\x01 \x01(\x0cR\x10intendedWorkerId\x12\x19\n\x08\x61\x63tor_id\x18\x02 \x01(\x0cR\x07\x61\x63torId\"\x1d\n\x1bWaitForActorOutOfScopeReply\"\x83\x01\n\x1d\x41\x64\x64ObjectLocationOwnerRequest\x12,\n\x12intended_worker_id\x18\x01 \x01(\x0cR\x10intendedWorkerId\x12\x1b\n\tobject_id\x18\x02 \x01(\x0cR\x08objectId\x12\x17\n\x07node_id\x18\x03 \x01(\x0cR\x06nodeId\"\x1d\n\x1b\x41\x64\x64ObjectLocationOwnerReply\"\x86\x01\n RemoveObjectLocationOwnerRequest\x12,\n\x12intended_worker_id\x18\x01 \x01(\x0cR\x10intendedWorkerId\x12\x1b\n\tobject_id\x18\x02 \x01(\x0cR\x08objectId\x12\x17\n\x07node_id\x18\x03 \x01(\x0cR\x06nodeId\" \n\x1eRemoveObjectLocationOwnerReply\"\x82\x01\n\x1eGetObjectLocationsOwnerRequest\x12`\n\x17object_location_request\x18\x01 \x01(\x0b\x32(.ray.rpc.WorkerObjectLocationsSubMessageR\x15objectLocationRequest\"z\n\x1cGetObjectLocationsOwnerReply\x12Z\n\x14object_location_info\x18\x01 \x01(\x0b\x32(.ray.rpc.WorkerObjectLocationsPubMessageR\x12objectLocationInfo\"|\n\x10KillActorRequest\x12*\n\x11intended_actor_id\x18\x01 \x01(\x0cR\x0fintendedActorId\x12\x1d\n\nforce_kill\x18\x02 \x01(\x08R\tforceKill\x12\x1d\n\nno_restart\x18\x03 \x01(\x08R\tnoRestart\"\x10\n\x0eKillActorReply\"z\n\x11\x43\x61ncelTaskRequest\x12(\n\x10intended_task_id\x18\x01 \x01(\x0cR\x0eintendedTaskId\x12\x1d\n\nforce_kill\x18\x02 \x01(\x08R\tforceKill\x12\x1c\n\trecursive\x18\x03 \x01(\x08R\trecursive\">\n\x0f\x43\x61ncelTaskReply\x12+\n\x11\x61ttempt_succeeded\x18\x01 \x01(\x08R\x10\x61ttemptSucceeded\"\x80\x01\n\x17RemoteCancelTaskRequest\x12(\n\x10remote_object_id\x18\x01 \x01(\x0cR\x0eremoteObjectId\x12\x1d\n\nforce_kill\x18\x02 \x01(\x08R\tforceKill\x12\x1c\n\trecursive\x18\x03 \x01(\x08R\trecursive\"\x17\n\x15RemoteCancelTaskReply\"y\n\x19GetCoreWorkerStatsRequest\x12,\n\x12intended_worker_id\x18\x01 \x01(\x0cR\x10intendedWorkerId\x12.\n\x13include_memory_info\x18\x02 \x01(\x08R\x11includeMemoryInfo\"_\n\x17GetCoreWorkerStatsReply\x12\x44\n\x11\x63ore_worker_stats\x18\x01 \x01(\x0b\x32\x18.ray.rpc.CoreWorkerStatsR\x0f\x63oreWorkerStats\"\x10\n\x0eLocalGCRequest\"\x0e\n\x0cLocalGCReply\"7\n\x18PlasmaObjectReadyRequest\x12\x1b\n\tobject_id\x18\x01 \x01(\x0cR\x08objectId\"\x18\n\x16PlasmaObjectReadyReply\"\x7f\n\x13SpillObjectsRequest\x12-\n\x13object_ids_to_spill\x18\x01 \x03(\x0cR\x10objectIdsToSpill\x12\x39\n\x0fowner_addresses\x18\x02 \x03(\x0b\x32\x10.ray.rpc.AddressR\x0eownerAddresses\"C\n\x11SpillObjectsReply\x12.\n\x13spilled_objects_url\x18\x01 \x03(\tR\x11spilledObjectsUrl\"\x81\x01\n\x1cRestoreSpilledObjectsRequest\x12.\n\x13spilled_objects_url\x18\x01 \x03(\tR\x11spilledObjectsUrl\x12\x31\n\x15object_ids_to_restore\x18\x02 \x03(\x0cR\x12objectIdsToRestore\"N\n\x1aRestoreSpilledObjectsReply\x12\x30\n\x14\x62ytes_restored_total\x18\x01 \x01(\x03R\x12\x62ytesRestoredTotal\"M\n\x1b\x44\x65leteSpilledObjectsRequest\x12.\n\x13spilled_objects_url\x18\x01 \x03(\tR\x11spilledObjectsUrl\"\x1b\n\x19\x44\x65leteSpilledObjectsReply\"\x90\x01\n\x14\x41\x64\x64SpilledUrlRequest\x12\x1b\n\tobject_id\x18\x01 \x01(\x0cR\x08objectId\x12\x1f\n\x0bspilled_url\x18\x02 \x01(\tR\nspilledUrl\x12&\n\x0fspilled_node_id\x18\x03 \x01(\x0cR\rspilledNodeId\x12\x12\n\x04size\x18\x04 \x01(\x03R\x04size\"\x14\n\x12\x41\x64\x64SpilledUrlReply\"\r\n\x0b\x45xitRequest\"%\n\tExitReply\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\"F\n\x16RunOnUtilWorkerRequest\x12\x18\n\x07request\x18\x01 \x01(\tR\x07request\x12\x12\n\x04\x61rgs\x18\x02 \x03(\tR\x04\x61rgs\"\x16\n\x14RunOnUtilWorkerReply\"\xe4\x01\n\x18\x41ssignObjectOwnerRequest\x12\x1b\n\tobject_id\x18\x01 \x01(\x0cR\x08objectId\x12\x1f\n\x0bobject_size\x18\x02 \x01(\x04R\nobjectSize\x12\x30\n\x14\x63ontained_object_ids\x18\x03 \x03(\x0cR\x12\x63ontainedObjectIds\x12;\n\x10\x62orrower_address\x18\x04 \x01(\x0b\x32\x10.ray.rpc.AddressR\x0f\x62orrowerAddress\x12\x1b\n\tcall_site\x18\x05 \x01(\tR\x08\x63\x61llSite\"\x18\n\x16\x41ssignObjectOwnerReply2\xd0\x0f\n\x11\x43oreWorkerService\x12<\n\x08PushTask\x12\x18.ray.rpc.PushTaskRequest\x1a\x16.ray.rpc.PushTaskReply\x12\x42\n\nStealTasks\x12\x1a.ray.rpc.StealTasksRequest\x1a\x18.ray.rpc.StealTasksReply\x12~\n\x1e\x44irectActorCallArgWaitComplete\x12..ray.rpc.DirectActorCallArgWaitCompleteRequest\x1a,.ray.rpc.DirectActorCallArgWaitCompleteReply\x12Q\n\x0fGetObjectStatus\x12\x1f.ray.rpc.GetObjectStatusRequest\x1a\x1d.ray.rpc.GetObjectStatusReply\x12\x66\n\x16WaitForActorOutOfScope\x12&.ray.rpc.WaitForActorOutOfScopeRequest\x1a$.ray.rpc.WaitForActorOutOfScopeReply\x12W\n\x11PubsubLongPolling\x12!.ray.rpc.PubsubLongPollingRequest\x1a\x1f.ray.rpc.PubsubLongPollingReply\x12Z\n\x12PubsubCommandBatch\x12\".ray.rpc.PubsubCommandBatchRequest\x1a .ray.rpc.PubsubCommandBatchReply\x12\x66\n\x16\x41\x64\x64ObjectLocationOwner\x12&.ray.rpc.AddObjectLocationOwnerRequest\x1a$.ray.rpc.AddObjectLocationOwnerReply\x12o\n\x19RemoveObjectLocationOwner\x12).ray.rpc.RemoveObjectLocationOwnerRequest\x1a\'.ray.rpc.RemoveObjectLocationOwnerReply\x12i\n\x17GetObjectLocationsOwner\x12\'.ray.rpc.GetObjectLocationsOwnerRequest\x1a%.ray.rpc.GetObjectLocationsOwnerReply\x12?\n\tKillActor\x12\x19.ray.rpc.KillActorRequest\x1a\x17.ray.rpc.KillActorReply\x12\x42\n\nCancelTask\x12\x1a.ray.rpc.CancelTaskRequest\x1a\x18.ray.rpc.CancelTaskReply\x12T\n\x10RemoteCancelTask\x12 .ray.rpc.RemoteCancelTaskRequest\x1a\x1e.ray.rpc.RemoteCancelTaskReply\x12Z\n\x12GetCoreWorkerStats\x12\".ray.rpc.GetCoreWorkerStatsRequest\x1a .ray.rpc.GetCoreWorkerStatsReply\x12\x39\n\x07LocalGC\x12\x17.ray.rpc.LocalGCRequest\x1a\x15.ray.rpc.LocalGCReply\x12H\n\x0cSpillObjects\x12\x1c.ray.rpc.SpillObjectsRequest\x1a\x1a.ray.rpc.SpillObjectsReply\x12\x63\n\x15RestoreSpilledObjects\x12%.ray.rpc.RestoreSpilledObjectsRequest\x1a#.ray.rpc.RestoreSpilledObjectsReply\x12`\n\x14\x44\x65leteSpilledObjects\x12$.ray.rpc.DeleteSpilledObjectsRequest\x1a\".ray.rpc.DeleteSpilledObjectsReply\x12K\n\rAddSpilledUrl\x12\x1d.ray.rpc.AddSpilledUrlRequest\x1a\x1b.ray.rpc.AddSpilledUrlReply\x12W\n\x11PlasmaObjectReady\x12!.ray.rpc.PlasmaObjectReadyRequest\x1a\x1f.ray.rpc.PlasmaObjectReadyReply\x12Q\n\x0fRunOnUtilWorker\x12\x1f.ray.rpc.RunOnUtilWorkerRequest\x1a\x1d.ray.rpc.RunOnUtilWorkerReply\x12\x30\n\x04\x45xit\x12\x14.ray.rpc.ExitRequest\x1a\x12.ray.rpc.ExitReply\x12W\n\x11\x41ssignObjectOwner\x12!.ray.rpc.AssignObjectOwnerRequest\x1a\x1f.ray.rpc.AssignObjectOwnerReplyb\x06proto3'
  ,
  dependencies=[src_dot_ray_dot_protobuf_dot_common__pb2.DESCRIPTOR,src_dot_ray_dot_protobuf_dot_pubsub__pb2.DESCRIPTOR,])



_GETOBJECTSTATUSREPLY_OBJECTSTATUS = _descriptor.EnumDescriptor(
  name='ObjectStatus',
  full_name='ray.rpc.GetObjectStatusReply.ObjectStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CREATED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OUT_OF_SCOPE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FREED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1974,
  serialized_end=2030,
)
_sym_db.RegisterEnumDescriptor(_GETOBJECTSTATUSREPLY_OBJECTSTATUS)


_ACTIVEOBJECTIDS = _descriptor.Descriptor(
  name='ActiveObjectIDs',
  full_name='ray.rpc.ActiveObjectIDs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_ids', full_name='ray.rpc.ActiveObjectIDs.object_ids', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=109,
  serialized_end=157,
)


_ACTORHANDLE = _descriptor.Descriptor(
  name='ActorHandle',
  full_name='ray.rpc.ActorHandle',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='ray.rpc.ActorHandle.actor_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='actorId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='owner_id', full_name='ray.rpc.ActorHandle.owner_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ownerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='owner_address', full_name='ray.rpc.ActorHandle.owner_address', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ownerAddress', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='creation_job_id', full_name='ray.rpc.ActorHandle.creation_job_id', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='creationJobId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_language', full_name='ray.rpc.ActorHandle.actor_language', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='actorLanguage', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_creation_task_function_descriptor', full_name='ray.rpc.ActorHandle.actor_creation_task_function_descriptor', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='actorCreationTaskFunctionDescriptor', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_cursor', full_name='ray.rpc.ActorHandle.actor_cursor', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='actorCursor', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='extension_data', full_name='ray.rpc.ActorHandle.extension_data', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='extensionData', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_task_retries', full_name='ray.rpc.ActorHandle.max_task_retries', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='maxTaskRetries', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=160,
  serialized_end=611,
)


_RETURNOBJECT = _descriptor.Descriptor(
  name='ReturnObject',
  full_name='ray.rpc.ReturnObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.ReturnObject.object_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='in_plasma', full_name='ray.rpc.ReturnObject.in_plasma', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inPlasma', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='ray.rpc.ReturnObject.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='data', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='ray.rpc.ReturnObject.metadata', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nested_inlined_ids', full_name='ray.rpc.ReturnObject.nested_inlined_ids', index=4,
      number=5, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='nestedInlinedIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='ray.rpc.ReturnObject.size', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='size', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=614,
  serialized_end=800,
)


_STEALTASKSREQUEST = _descriptor.Descriptor(
  name='StealTasksRequest',
  full_name='ray.rpc.StealTasksRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='thief_addr', full_name='ray.rpc.StealTasksRequest.thief_addr', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thiefAddr', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=802,
  serialized_end=870,
)


_STEALTASKSREPLY = _descriptor.Descriptor(
  name='StealTasksReply',
  full_name='ray.rpc.StealTasksReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='stolen_tasks_ids', full_name='ray.rpc.StealTasksReply.stolen_tasks_ids', index=0,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='stolenTasksIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=872,
  serialized_end=931,
)


_PUSHTASKREQUEST = _descriptor.Descriptor(
  name='PushTaskRequest',
  full_name='ray.rpc.PushTaskRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_worker_id', full_name='ray.rpc.PushTaskRequest.intended_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='task_spec', full_name='ray.rpc.PushTaskRequest.task_spec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='taskSpec', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sequence_number', full_name='ray.rpc.PushTaskRequest.sequence_number', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sequenceNumber', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_processed_up_to', full_name='ray.rpc.PushTaskRequest.client_processed_up_to', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clientProcessedUpTo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_mapping', full_name='ray.rpc.PushTaskRequest.resource_mapping', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceMapping', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=934,
  serialized_end=1209,
)


_PUSHTASKREPLY = _descriptor.Descriptor(
  name='PushTaskReply',
  full_name='ray.rpc.PushTaskReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_objects', full_name='ray.rpc.PushTaskReply.return_objects', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='returnObjects', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='task_stolen', full_name='ray.rpc.PushTaskReply.task_stolen', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='taskStolen', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='worker_exiting', full_name='ray.rpc.PushTaskReply.worker_exiting', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workerExiting', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='borrowed_refs', full_name='ray.rpc.PushTaskReply.borrowed_refs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='borrowedRefs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1212,
  serialized_end=1429,
)


_DIRECTACTORCALLARGWAITCOMPLETEREQUEST = _descriptor.Descriptor(
  name='DirectActorCallArgWaitCompleteRequest',
  full_name='ray.rpc.DirectActorCallArgWaitCompleteRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_worker_id', full_name='ray.rpc.DirectActorCallArgWaitCompleteRequest.intended_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag', full_name='ray.rpc.DirectActorCallArgWaitCompleteRequest.tag', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='tag', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1431,
  serialized_end=1534,
)


_DIRECTACTORCALLARGWAITCOMPLETEREPLY = _descriptor.Descriptor(
  name='DirectActorCallArgWaitCompleteReply',
  full_name='ray.rpc.DirectActorCallArgWaitCompleteReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1536,
  serialized_end=1573,
)


_GETOBJECTSTATUSREQUEST = _descriptor.Descriptor(
  name='GetObjectStatusRequest',
  full_name='ray.rpc.GetObjectStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner_worker_id', full_name='ray.rpc.GetObjectStatusRequest.owner_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ownerWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.GetObjectStatusRequest.object_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1575,
  serialized_end=1668,
)


_RAYOBJECT = _descriptor.Descriptor(
  name='RayObject',
  full_name='ray.rpc.RayObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='ray.rpc.RayObject.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='data', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='ray.rpc.RayObject.metadata', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nested_inlined_ids', full_name='ray.rpc.RayObject.nested_inlined_ids', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='nestedInlinedIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1670,
  serialized_end=1775,
)


_GETOBJECTSTATUSREPLY = _descriptor.Descriptor(
  name='GetObjectStatusReply',
  full_name='ray.rpc.GetObjectStatusReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='ray.rpc.GetObjectStatusReply.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object', full_name='ray.rpc.GetObjectStatusReply.object', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='object', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='node_ids', full_name='ray.rpc.GetObjectStatusReply.node_ids', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='nodeIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_size', full_name='ray.rpc.GetObjectStatusReply.object_size', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectSize', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETOBJECTSTATUSREPLY_OBJECTSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1778,
  serialized_end=2030,
)


_WAITFORACTOROUTOFSCOPEREQUEST = _descriptor.Descriptor(
  name='WaitForActorOutOfScopeRequest',
  full_name='ray.rpc.WaitForActorOutOfScopeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_worker_id', full_name='ray.rpc.WaitForActorOutOfScopeRequest.intended_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_id', full_name='ray.rpc.WaitForActorOutOfScopeRequest.actor_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='actorId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2032,
  serialized_end=2136,
)


_WAITFORACTOROUTOFSCOPEREPLY = _descriptor.Descriptor(
  name='WaitForActorOutOfScopeReply',
  full_name='ray.rpc.WaitForActorOutOfScopeReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2138,
  serialized_end=2167,
)


_ADDOBJECTLOCATIONOWNERREQUEST = _descriptor.Descriptor(
  name='AddObjectLocationOwnerRequest',
  full_name='ray.rpc.AddObjectLocationOwnerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_worker_id', full_name='ray.rpc.AddObjectLocationOwnerRequest.intended_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.AddObjectLocationOwnerRequest.object_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='node_id', full_name='ray.rpc.AddObjectLocationOwnerRequest.node_id', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='nodeId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2170,
  serialized_end=2301,
)


_ADDOBJECTLOCATIONOWNERREPLY = _descriptor.Descriptor(
  name='AddObjectLocationOwnerReply',
  full_name='ray.rpc.AddObjectLocationOwnerReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2303,
  serialized_end=2332,
)


_REMOVEOBJECTLOCATIONOWNERREQUEST = _descriptor.Descriptor(
  name='RemoveObjectLocationOwnerRequest',
  full_name='ray.rpc.RemoveObjectLocationOwnerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_worker_id', full_name='ray.rpc.RemoveObjectLocationOwnerRequest.intended_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.RemoveObjectLocationOwnerRequest.object_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='node_id', full_name='ray.rpc.RemoveObjectLocationOwnerRequest.node_id', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='nodeId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2335,
  serialized_end=2469,
)


_REMOVEOBJECTLOCATIONOWNERREPLY = _descriptor.Descriptor(
  name='RemoveObjectLocationOwnerReply',
  full_name='ray.rpc.RemoveObjectLocationOwnerReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2471,
  serialized_end=2503,
)


_GETOBJECTLOCATIONSOWNERREQUEST = _descriptor.Descriptor(
  name='GetObjectLocationsOwnerRequest',
  full_name='ray.rpc.GetObjectLocationsOwnerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_location_request', full_name='ray.rpc.GetObjectLocationsOwnerRequest.object_location_request', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectLocationRequest', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2506,
  serialized_end=2636,
)


_GETOBJECTLOCATIONSOWNERREPLY = _descriptor.Descriptor(
  name='GetObjectLocationsOwnerReply',
  full_name='ray.rpc.GetObjectLocationsOwnerReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_location_info', full_name='ray.rpc.GetObjectLocationsOwnerReply.object_location_info', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectLocationInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2638,
  serialized_end=2760,
)


_KILLACTORREQUEST = _descriptor.Descriptor(
  name='KillActorRequest',
  full_name='ray.rpc.KillActorRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_actor_id', full_name='ray.rpc.KillActorRequest.intended_actor_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedActorId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='force_kill', full_name='ray.rpc.KillActorRequest.force_kill', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='forceKill', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='no_restart', full_name='ray.rpc.KillActorRequest.no_restart', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='noRestart', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2762,
  serialized_end=2886,
)


_KILLACTORREPLY = _descriptor.Descriptor(
  name='KillActorReply',
  full_name='ray.rpc.KillActorReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2888,
  serialized_end=2904,
)


_CANCELTASKREQUEST = _descriptor.Descriptor(
  name='CancelTaskRequest',
  full_name='ray.rpc.CancelTaskRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_task_id', full_name='ray.rpc.CancelTaskRequest.intended_task_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedTaskId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='force_kill', full_name='ray.rpc.CancelTaskRequest.force_kill', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='forceKill', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='ray.rpc.CancelTaskRequest.recursive', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='recursive', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2906,
  serialized_end=3028,
)


_CANCELTASKREPLY = _descriptor.Descriptor(
  name='CancelTaskReply',
  full_name='ray.rpc.CancelTaskReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='attempt_succeeded', full_name='ray.rpc.CancelTaskReply.attempt_succeeded', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attemptSucceeded', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3030,
  serialized_end=3092,
)


_REMOTECANCELTASKREQUEST = _descriptor.Descriptor(
  name='RemoteCancelTaskRequest',
  full_name='ray.rpc.RemoteCancelTaskRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='remote_object_id', full_name='ray.rpc.RemoteCancelTaskRequest.remote_object_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='remoteObjectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='force_kill', full_name='ray.rpc.RemoteCancelTaskRequest.force_kill', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='forceKill', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='ray.rpc.RemoteCancelTaskRequest.recursive', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='recursive', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3095,
  serialized_end=3223,
)


_REMOTECANCELTASKREPLY = _descriptor.Descriptor(
  name='RemoteCancelTaskReply',
  full_name='ray.rpc.RemoteCancelTaskReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3225,
  serialized_end=3248,
)


_GETCOREWORKERSTATSREQUEST = _descriptor.Descriptor(
  name='GetCoreWorkerStatsRequest',
  full_name='ray.rpc.GetCoreWorkerStatsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intended_worker_id', full_name='ray.rpc.GetCoreWorkerStatsRequest.intended_worker_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='intendedWorkerId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='include_memory_info', full_name='ray.rpc.GetCoreWorkerStatsRequest.include_memory_info', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='includeMemoryInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3250,
  serialized_end=3371,
)


_GETCOREWORKERSTATSREPLY = _descriptor.Descriptor(
  name='GetCoreWorkerStatsReply',
  full_name='ray.rpc.GetCoreWorkerStatsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='core_worker_stats', full_name='ray.rpc.GetCoreWorkerStatsReply.core_worker_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='coreWorkerStats', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3373,
  serialized_end=3468,
)


_LOCALGCREQUEST = _descriptor.Descriptor(
  name='LocalGCRequest',
  full_name='ray.rpc.LocalGCRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3470,
  serialized_end=3486,
)


_LOCALGCREPLY = _descriptor.Descriptor(
  name='LocalGCReply',
  full_name='ray.rpc.LocalGCReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3488,
  serialized_end=3502,
)


_PLASMAOBJECTREADYREQUEST = _descriptor.Descriptor(
  name='PlasmaObjectReadyRequest',
  full_name='ray.rpc.PlasmaObjectReadyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.PlasmaObjectReadyRequest.object_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3504,
  serialized_end=3559,
)


_PLASMAOBJECTREADYREPLY = _descriptor.Descriptor(
  name='PlasmaObjectReadyReply',
  full_name='ray.rpc.PlasmaObjectReadyReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3561,
  serialized_end=3585,
)


_SPILLOBJECTSREQUEST = _descriptor.Descriptor(
  name='SpillObjectsRequest',
  full_name='ray.rpc.SpillObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_ids_to_spill', full_name='ray.rpc.SpillObjectsRequest.object_ids_to_spill', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectIdsToSpill', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='owner_addresses', full_name='ray.rpc.SpillObjectsRequest.owner_addresses', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ownerAddresses', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3587,
  serialized_end=3714,
)


_SPILLOBJECTSREPLY = _descriptor.Descriptor(
  name='SpillObjectsReply',
  full_name='ray.rpc.SpillObjectsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spilled_objects_url', full_name='ray.rpc.SpillObjectsReply.spilled_objects_url', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spilledObjectsUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3716,
  serialized_end=3783,
)


_RESTORESPILLEDOBJECTSREQUEST = _descriptor.Descriptor(
  name='RestoreSpilledObjectsRequest',
  full_name='ray.rpc.RestoreSpilledObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spilled_objects_url', full_name='ray.rpc.RestoreSpilledObjectsRequest.spilled_objects_url', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spilledObjectsUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_ids_to_restore', full_name='ray.rpc.RestoreSpilledObjectsRequest.object_ids_to_restore', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectIdsToRestore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3786,
  serialized_end=3915,
)


_RESTORESPILLEDOBJECTSREPLY = _descriptor.Descriptor(
  name='RestoreSpilledObjectsReply',
  full_name='ray.rpc.RestoreSpilledObjectsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bytes_restored_total', full_name='ray.rpc.RestoreSpilledObjectsReply.bytes_restored_total', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bytesRestoredTotal', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3917,
  serialized_end=3995,
)


_DELETESPILLEDOBJECTSREQUEST = _descriptor.Descriptor(
  name='DeleteSpilledObjectsRequest',
  full_name='ray.rpc.DeleteSpilledObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spilled_objects_url', full_name='ray.rpc.DeleteSpilledObjectsRequest.spilled_objects_url', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spilledObjectsUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3997,
  serialized_end=4074,
)


_DELETESPILLEDOBJECTSREPLY = _descriptor.Descriptor(
  name='DeleteSpilledObjectsReply',
  full_name='ray.rpc.DeleteSpilledObjectsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4076,
  serialized_end=4103,
)


_ADDSPILLEDURLREQUEST = _descriptor.Descriptor(
  name='AddSpilledUrlRequest',
  full_name='ray.rpc.AddSpilledUrlRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.AddSpilledUrlRequest.object_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spilled_url', full_name='ray.rpc.AddSpilledUrlRequest.spilled_url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spilledUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spilled_node_id', full_name='ray.rpc.AddSpilledUrlRequest.spilled_node_id', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spilledNodeId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='ray.rpc.AddSpilledUrlRequest.size', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='size', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4106,
  serialized_end=4250,
)


_ADDSPILLEDURLREPLY = _descriptor.Descriptor(
  name='AddSpilledUrlReply',
  full_name='ray.rpc.AddSpilledUrlReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4252,
  serialized_end=4272,
)


_EXITREQUEST = _descriptor.Descriptor(
  name='ExitRequest',
  full_name='ray.rpc.ExitRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4274,
  serialized_end=4287,
)


_EXITREPLY = _descriptor.Descriptor(
  name='ExitReply',
  full_name='ray.rpc.ExitReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ray.rpc.ExitReply.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='success', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4289,
  serialized_end=4326,
)


_RUNONUTILWORKERREQUEST = _descriptor.Descriptor(
  name='RunOnUtilWorkerRequest',
  full_name='ray.rpc.RunOnUtilWorkerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='request', full_name='ray.rpc.RunOnUtilWorkerRequest.request', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='request', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='ray.rpc.RunOnUtilWorkerRequest.args', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='args', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4328,
  serialized_end=4398,
)


_RUNONUTILWORKERREPLY = _descriptor.Descriptor(
  name='RunOnUtilWorkerReply',
  full_name='ray.rpc.RunOnUtilWorkerReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4400,
  serialized_end=4422,
)


_ASSIGNOBJECTOWNERREQUEST = _descriptor.Descriptor(
  name='AssignObjectOwnerRequest',
  full_name='ray.rpc.AssignObjectOwnerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.AssignObjectOwnerRequest.object_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_size', full_name='ray.rpc.AssignObjectOwnerRequest.object_size', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectSize', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contained_object_ids', full_name='ray.rpc.AssignObjectOwnerRequest.contained_object_ids', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='containedObjectIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='borrower_address', full_name='ray.rpc.AssignObjectOwnerRequest.borrower_address', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='borrowerAddress', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='call_site', full_name='ray.rpc.AssignObjectOwnerRequest.call_site', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='callSite', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4425,
  serialized_end=4653,
)


_ASSIGNOBJECTOWNERREPLY = _descriptor.Descriptor(
  name='AssignObjectOwnerReply',
  full_name='ray.rpc.AssignObjectOwnerReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4655,
  serialized_end=4679,
)

_ACTORHANDLE.fields_by_name['owner_address'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
_ACTORHANDLE.fields_by_name['actor_language'].enum_type = src_dot_ray_dot_protobuf_dot_common__pb2._LANGUAGE
_ACTORHANDLE.fields_by_name['actor_creation_task_function_descriptor'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._FUNCTIONDESCRIPTOR
_STEALTASKSREQUEST.fields_by_name['thief_addr'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
_PUSHTASKREQUEST.fields_by_name['task_spec'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._TASKSPEC
_PUSHTASKREQUEST.fields_by_name['resource_mapping'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._RESOURCEMAPENTRY
_PUSHTASKREPLY.fields_by_name['return_objects'].message_type = _RETURNOBJECT
_PUSHTASKREPLY.fields_by_name['borrowed_refs'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._OBJECTREFERENCECOUNT
_GETOBJECTSTATUSREPLY.fields_by_name['status'].enum_type = _GETOBJECTSTATUSREPLY_OBJECTSTATUS
_GETOBJECTSTATUSREPLY.fields_by_name['object'].message_type = _RAYOBJECT
_GETOBJECTSTATUSREPLY_OBJECTSTATUS.containing_type = _GETOBJECTSTATUSREPLY
_GETOBJECTLOCATIONSOWNERREQUEST.fields_by_name['object_location_request'].message_type = src_dot_ray_dot_protobuf_dot_pubsub__pb2._WORKEROBJECTLOCATIONSSUBMESSAGE
_GETOBJECTLOCATIONSOWNERREPLY.fields_by_name['object_location_info'].message_type = src_dot_ray_dot_protobuf_dot_pubsub__pb2._WORKEROBJECTLOCATIONSPUBMESSAGE
_GETCOREWORKERSTATSREPLY.fields_by_name['core_worker_stats'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._COREWORKERSTATS
_SPILLOBJECTSREQUEST.fields_by_name['owner_addresses'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
_ASSIGNOBJECTOWNERREQUEST.fields_by_name['borrower_address'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
DESCRIPTOR.message_types_by_name['ActiveObjectIDs'] = _ACTIVEOBJECTIDS
DESCRIPTOR.message_types_by_name['ActorHandle'] = _ACTORHANDLE
DESCRIPTOR.message_types_by_name['ReturnObject'] = _RETURNOBJECT
DESCRIPTOR.message_types_by_name['StealTasksRequest'] = _STEALTASKSREQUEST
DESCRIPTOR.message_types_by_name['StealTasksReply'] = _STEALTASKSREPLY
DESCRIPTOR.message_types_by_name['PushTaskRequest'] = _PUSHTASKREQUEST
DESCRIPTOR.message_types_by_name['PushTaskReply'] = _PUSHTASKREPLY
DESCRIPTOR.message_types_by_name['DirectActorCallArgWaitCompleteRequest'] = _DIRECTACTORCALLARGWAITCOMPLETEREQUEST
DESCRIPTOR.message_types_by_name['DirectActorCallArgWaitCompleteReply'] = _DIRECTACTORCALLARGWAITCOMPLETEREPLY
DESCRIPTOR.message_types_by_name['GetObjectStatusRequest'] = _GETOBJECTSTATUSREQUEST
DESCRIPTOR.message_types_by_name['RayObject'] = _RAYOBJECT
DESCRIPTOR.message_types_by_name['GetObjectStatusReply'] = _GETOBJECTSTATUSREPLY
DESCRIPTOR.message_types_by_name['WaitForActorOutOfScopeRequest'] = _WAITFORACTOROUTOFSCOPEREQUEST
DESCRIPTOR.message_types_by_name['WaitForActorOutOfScopeReply'] = _WAITFORACTOROUTOFSCOPEREPLY
DESCRIPTOR.message_types_by_name['AddObjectLocationOwnerRequest'] = _ADDOBJECTLOCATIONOWNERREQUEST
DESCRIPTOR.message_types_by_name['AddObjectLocationOwnerReply'] = _ADDOBJECTLOCATIONOWNERREPLY
DESCRIPTOR.message_types_by_name['RemoveObjectLocationOwnerRequest'] = _REMOVEOBJECTLOCATIONOWNERREQUEST
DESCRIPTOR.message_types_by_name['RemoveObjectLocationOwnerReply'] = _REMOVEOBJECTLOCATIONOWNERREPLY
DESCRIPTOR.message_types_by_name['GetObjectLocationsOwnerRequest'] = _GETOBJECTLOCATIONSOWNERREQUEST
DESCRIPTOR.message_types_by_name['GetObjectLocationsOwnerReply'] = _GETOBJECTLOCATIONSOWNERREPLY
DESCRIPTOR.message_types_by_name['KillActorRequest'] = _KILLACTORREQUEST
DESCRIPTOR.message_types_by_name['KillActorReply'] = _KILLACTORREPLY
DESCRIPTOR.message_types_by_name['CancelTaskRequest'] = _CANCELTASKREQUEST
DESCRIPTOR.message_types_by_name['CancelTaskReply'] = _CANCELTASKREPLY
DESCRIPTOR.message_types_by_name['RemoteCancelTaskRequest'] = _REMOTECANCELTASKREQUEST
DESCRIPTOR.message_types_by_name['RemoteCancelTaskReply'] = _REMOTECANCELTASKREPLY
DESCRIPTOR.message_types_by_name['GetCoreWorkerStatsRequest'] = _GETCOREWORKERSTATSREQUEST
DESCRIPTOR.message_types_by_name['GetCoreWorkerStatsReply'] = _GETCOREWORKERSTATSREPLY
DESCRIPTOR.message_types_by_name['LocalGCRequest'] = _LOCALGCREQUEST
DESCRIPTOR.message_types_by_name['LocalGCReply'] = _LOCALGCREPLY
DESCRIPTOR.message_types_by_name['PlasmaObjectReadyRequest'] = _PLASMAOBJECTREADYREQUEST
DESCRIPTOR.message_types_by_name['PlasmaObjectReadyReply'] = _PLASMAOBJECTREADYREPLY
DESCRIPTOR.message_types_by_name['SpillObjectsRequest'] = _SPILLOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['SpillObjectsReply'] = _SPILLOBJECTSREPLY
DESCRIPTOR.message_types_by_name['RestoreSpilledObjectsRequest'] = _RESTORESPILLEDOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['RestoreSpilledObjectsReply'] = _RESTORESPILLEDOBJECTSREPLY
DESCRIPTOR.message_types_by_name['DeleteSpilledObjectsRequest'] = _DELETESPILLEDOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['DeleteSpilledObjectsReply'] = _DELETESPILLEDOBJECTSREPLY
DESCRIPTOR.message_types_by_name['AddSpilledUrlRequest'] = _ADDSPILLEDURLREQUEST
DESCRIPTOR.message_types_by_name['AddSpilledUrlReply'] = _ADDSPILLEDURLREPLY
DESCRIPTOR.message_types_by_name['ExitRequest'] = _EXITREQUEST
DESCRIPTOR.message_types_by_name['ExitReply'] = _EXITREPLY
DESCRIPTOR.message_types_by_name['RunOnUtilWorkerRequest'] = _RUNONUTILWORKERREQUEST
DESCRIPTOR.message_types_by_name['RunOnUtilWorkerReply'] = _RUNONUTILWORKERREPLY
DESCRIPTOR.message_types_by_name['AssignObjectOwnerRequest'] = _ASSIGNOBJECTOWNERREQUEST
DESCRIPTOR.message_types_by_name['AssignObjectOwnerReply'] = _ASSIGNOBJECTOWNERREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ActiveObjectIDs = _reflection.GeneratedProtocolMessageType('ActiveObjectIDs', (_message.Message,), {
  'DESCRIPTOR' : _ACTIVEOBJECTIDS,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ActiveObjectIDs)
  })
_sym_db.RegisterMessage(ActiveObjectIDs)

ActorHandle = _reflection.GeneratedProtocolMessageType('ActorHandle', (_message.Message,), {
  'DESCRIPTOR' : _ACTORHANDLE,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ActorHandle)
  })
_sym_db.RegisterMessage(ActorHandle)

ReturnObject = _reflection.GeneratedProtocolMessageType('ReturnObject', (_message.Message,), {
  'DESCRIPTOR' : _RETURNOBJECT,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReturnObject)
  })
_sym_db.RegisterMessage(ReturnObject)

StealTasksRequest = _reflection.GeneratedProtocolMessageType('StealTasksRequest', (_message.Message,), {
  'DESCRIPTOR' : _STEALTASKSREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.StealTasksRequest)
  })
_sym_db.RegisterMessage(StealTasksRequest)

StealTasksReply = _reflection.GeneratedProtocolMessageType('StealTasksReply', (_message.Message,), {
  'DESCRIPTOR' : _STEALTASKSREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.StealTasksReply)
  })
_sym_db.RegisterMessage(StealTasksReply)

PushTaskRequest = _reflection.GeneratedProtocolMessageType('PushTaskRequest', (_message.Message,), {
  'DESCRIPTOR' : _PUSHTASKREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PushTaskRequest)
  })
_sym_db.RegisterMessage(PushTaskRequest)

PushTaskReply = _reflection.GeneratedProtocolMessageType('PushTaskReply', (_message.Message,), {
  'DESCRIPTOR' : _PUSHTASKREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PushTaskReply)
  })
_sym_db.RegisterMessage(PushTaskReply)

DirectActorCallArgWaitCompleteRequest = _reflection.GeneratedProtocolMessageType('DirectActorCallArgWaitCompleteRequest', (_message.Message,), {
  'DESCRIPTOR' : _DIRECTACTORCALLARGWAITCOMPLETEREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.DirectActorCallArgWaitCompleteRequest)
  })
_sym_db.RegisterMessage(DirectActorCallArgWaitCompleteRequest)

DirectActorCallArgWaitCompleteReply = _reflection.GeneratedProtocolMessageType('DirectActorCallArgWaitCompleteReply', (_message.Message,), {
  'DESCRIPTOR' : _DIRECTACTORCALLARGWAITCOMPLETEREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.DirectActorCallArgWaitCompleteReply)
  })
_sym_db.RegisterMessage(DirectActorCallArgWaitCompleteReply)

GetObjectStatusRequest = _reflection.GeneratedProtocolMessageType('GetObjectStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETOBJECTSTATUSREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetObjectStatusRequest)
  })
_sym_db.RegisterMessage(GetObjectStatusRequest)

RayObject = _reflection.GeneratedProtocolMessageType('RayObject', (_message.Message,), {
  'DESCRIPTOR' : _RAYOBJECT,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RayObject)
  })
_sym_db.RegisterMessage(RayObject)

GetObjectStatusReply = _reflection.GeneratedProtocolMessageType('GetObjectStatusReply', (_message.Message,), {
  'DESCRIPTOR' : _GETOBJECTSTATUSREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetObjectStatusReply)
  })
_sym_db.RegisterMessage(GetObjectStatusReply)

WaitForActorOutOfScopeRequest = _reflection.GeneratedProtocolMessageType('WaitForActorOutOfScopeRequest', (_message.Message,), {
  'DESCRIPTOR' : _WAITFORACTOROUTOFSCOPEREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.WaitForActorOutOfScopeRequest)
  })
_sym_db.RegisterMessage(WaitForActorOutOfScopeRequest)

WaitForActorOutOfScopeReply = _reflection.GeneratedProtocolMessageType('WaitForActorOutOfScopeReply', (_message.Message,), {
  'DESCRIPTOR' : _WAITFORACTOROUTOFSCOPEREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.WaitForActorOutOfScopeReply)
  })
_sym_db.RegisterMessage(WaitForActorOutOfScopeReply)

AddObjectLocationOwnerRequest = _reflection.GeneratedProtocolMessageType('AddObjectLocationOwnerRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDOBJECTLOCATIONOWNERREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.AddObjectLocationOwnerRequest)
  })
_sym_db.RegisterMessage(AddObjectLocationOwnerRequest)

AddObjectLocationOwnerReply = _reflection.GeneratedProtocolMessageType('AddObjectLocationOwnerReply', (_message.Message,), {
  'DESCRIPTOR' : _ADDOBJECTLOCATIONOWNERREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.AddObjectLocationOwnerReply)
  })
_sym_db.RegisterMessage(AddObjectLocationOwnerReply)

RemoveObjectLocationOwnerRequest = _reflection.GeneratedProtocolMessageType('RemoveObjectLocationOwnerRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEOBJECTLOCATIONOWNERREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RemoveObjectLocationOwnerRequest)
  })
_sym_db.RegisterMessage(RemoveObjectLocationOwnerRequest)

RemoveObjectLocationOwnerReply = _reflection.GeneratedProtocolMessageType('RemoveObjectLocationOwnerReply', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEOBJECTLOCATIONOWNERREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RemoveObjectLocationOwnerReply)
  })
_sym_db.RegisterMessage(RemoveObjectLocationOwnerReply)

GetObjectLocationsOwnerRequest = _reflection.GeneratedProtocolMessageType('GetObjectLocationsOwnerRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETOBJECTLOCATIONSOWNERREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetObjectLocationsOwnerRequest)
  })
_sym_db.RegisterMessage(GetObjectLocationsOwnerRequest)

GetObjectLocationsOwnerReply = _reflection.GeneratedProtocolMessageType('GetObjectLocationsOwnerReply', (_message.Message,), {
  'DESCRIPTOR' : _GETOBJECTLOCATIONSOWNERREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetObjectLocationsOwnerReply)
  })
_sym_db.RegisterMessage(GetObjectLocationsOwnerReply)

KillActorRequest = _reflection.GeneratedProtocolMessageType('KillActorRequest', (_message.Message,), {
  'DESCRIPTOR' : _KILLACTORREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.KillActorRequest)
  })
_sym_db.RegisterMessage(KillActorRequest)

KillActorReply = _reflection.GeneratedProtocolMessageType('KillActorReply', (_message.Message,), {
  'DESCRIPTOR' : _KILLACTORREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.KillActorReply)
  })
_sym_db.RegisterMessage(KillActorReply)

CancelTaskRequest = _reflection.GeneratedProtocolMessageType('CancelTaskRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELTASKREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CancelTaskRequest)
  })
_sym_db.RegisterMessage(CancelTaskRequest)

CancelTaskReply = _reflection.GeneratedProtocolMessageType('CancelTaskReply', (_message.Message,), {
  'DESCRIPTOR' : _CANCELTASKREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CancelTaskReply)
  })
_sym_db.RegisterMessage(CancelTaskReply)

RemoteCancelTaskRequest = _reflection.GeneratedProtocolMessageType('RemoteCancelTaskRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOTECANCELTASKREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RemoteCancelTaskRequest)
  })
_sym_db.RegisterMessage(RemoteCancelTaskRequest)

RemoteCancelTaskReply = _reflection.GeneratedProtocolMessageType('RemoteCancelTaskReply', (_message.Message,), {
  'DESCRIPTOR' : _REMOTECANCELTASKREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RemoteCancelTaskReply)
  })
_sym_db.RegisterMessage(RemoteCancelTaskReply)

GetCoreWorkerStatsRequest = _reflection.GeneratedProtocolMessageType('GetCoreWorkerStatsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETCOREWORKERSTATSREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetCoreWorkerStatsRequest)
  })
_sym_db.RegisterMessage(GetCoreWorkerStatsRequest)

GetCoreWorkerStatsReply = _reflection.GeneratedProtocolMessageType('GetCoreWorkerStatsReply', (_message.Message,), {
  'DESCRIPTOR' : _GETCOREWORKERSTATSREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetCoreWorkerStatsReply)
  })
_sym_db.RegisterMessage(GetCoreWorkerStatsReply)

LocalGCRequest = _reflection.GeneratedProtocolMessageType('LocalGCRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOCALGCREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.LocalGCRequest)
  })
_sym_db.RegisterMessage(LocalGCRequest)

LocalGCReply = _reflection.GeneratedProtocolMessageType('LocalGCReply', (_message.Message,), {
  'DESCRIPTOR' : _LOCALGCREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.LocalGCReply)
  })
_sym_db.RegisterMessage(LocalGCReply)

PlasmaObjectReadyRequest = _reflection.GeneratedProtocolMessageType('PlasmaObjectReadyRequest', (_message.Message,), {
  'DESCRIPTOR' : _PLASMAOBJECTREADYREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PlasmaObjectReadyRequest)
  })
_sym_db.RegisterMessage(PlasmaObjectReadyRequest)

PlasmaObjectReadyReply = _reflection.GeneratedProtocolMessageType('PlasmaObjectReadyReply', (_message.Message,), {
  'DESCRIPTOR' : _PLASMAOBJECTREADYREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PlasmaObjectReadyReply)
  })
_sym_db.RegisterMessage(PlasmaObjectReadyReply)

SpillObjectsRequest = _reflection.GeneratedProtocolMessageType('SpillObjectsRequest', (_message.Message,), {
  'DESCRIPTOR' : _SPILLOBJECTSREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.SpillObjectsRequest)
  })
_sym_db.RegisterMessage(SpillObjectsRequest)

SpillObjectsReply = _reflection.GeneratedProtocolMessageType('SpillObjectsReply', (_message.Message,), {
  'DESCRIPTOR' : _SPILLOBJECTSREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.SpillObjectsReply)
  })
_sym_db.RegisterMessage(SpillObjectsReply)

RestoreSpilledObjectsRequest = _reflection.GeneratedProtocolMessageType('RestoreSpilledObjectsRequest', (_message.Message,), {
  'DESCRIPTOR' : _RESTORESPILLEDOBJECTSREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RestoreSpilledObjectsRequest)
  })
_sym_db.RegisterMessage(RestoreSpilledObjectsRequest)

RestoreSpilledObjectsReply = _reflection.GeneratedProtocolMessageType('RestoreSpilledObjectsReply', (_message.Message,), {
  'DESCRIPTOR' : _RESTORESPILLEDOBJECTSREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RestoreSpilledObjectsReply)
  })
_sym_db.RegisterMessage(RestoreSpilledObjectsReply)

DeleteSpilledObjectsRequest = _reflection.GeneratedProtocolMessageType('DeleteSpilledObjectsRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETESPILLEDOBJECTSREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.DeleteSpilledObjectsRequest)
  })
_sym_db.RegisterMessage(DeleteSpilledObjectsRequest)

DeleteSpilledObjectsReply = _reflection.GeneratedProtocolMessageType('DeleteSpilledObjectsReply', (_message.Message,), {
  'DESCRIPTOR' : _DELETESPILLEDOBJECTSREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.DeleteSpilledObjectsReply)
  })
_sym_db.RegisterMessage(DeleteSpilledObjectsReply)

AddSpilledUrlRequest = _reflection.GeneratedProtocolMessageType('AddSpilledUrlRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDSPILLEDURLREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.AddSpilledUrlRequest)
  })
_sym_db.RegisterMessage(AddSpilledUrlRequest)

AddSpilledUrlReply = _reflection.GeneratedProtocolMessageType('AddSpilledUrlReply', (_message.Message,), {
  'DESCRIPTOR' : _ADDSPILLEDURLREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.AddSpilledUrlReply)
  })
_sym_db.RegisterMessage(AddSpilledUrlReply)

ExitRequest = _reflection.GeneratedProtocolMessageType('ExitRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXITREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ExitRequest)
  })
_sym_db.RegisterMessage(ExitRequest)

ExitReply = _reflection.GeneratedProtocolMessageType('ExitReply', (_message.Message,), {
  'DESCRIPTOR' : _EXITREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ExitReply)
  })
_sym_db.RegisterMessage(ExitReply)

RunOnUtilWorkerRequest = _reflection.GeneratedProtocolMessageType('RunOnUtilWorkerRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNONUTILWORKERREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RunOnUtilWorkerRequest)
  })
_sym_db.RegisterMessage(RunOnUtilWorkerRequest)

RunOnUtilWorkerReply = _reflection.GeneratedProtocolMessageType('RunOnUtilWorkerReply', (_message.Message,), {
  'DESCRIPTOR' : _RUNONUTILWORKERREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RunOnUtilWorkerReply)
  })
_sym_db.RegisterMessage(RunOnUtilWorkerReply)

AssignObjectOwnerRequest = _reflection.GeneratedProtocolMessageType('AssignObjectOwnerRequest', (_message.Message,), {
  'DESCRIPTOR' : _ASSIGNOBJECTOWNERREQUEST,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.AssignObjectOwnerRequest)
  })
_sym_db.RegisterMessage(AssignObjectOwnerRequest)

AssignObjectOwnerReply = _reflection.GeneratedProtocolMessageType('AssignObjectOwnerReply', (_message.Message,), {
  'DESCRIPTOR' : _ASSIGNOBJECTOWNERREPLY,
  '__module__' : 'src.ray.protobuf.core_worker_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.AssignObjectOwnerReply)
  })
_sym_db.RegisterMessage(AssignObjectOwnerReply)



_COREWORKERSERVICE = _descriptor.ServiceDescriptor(
  name='CoreWorkerService',
  full_name='ray.rpc.CoreWorkerService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=4682,
  serialized_end=6682,
  methods=[
  _descriptor.MethodDescriptor(
    name='PushTask',
    full_name='ray.rpc.CoreWorkerService.PushTask',
    index=0,
    containing_service=None,
    input_type=_PUSHTASKREQUEST,
    output_type=_PUSHTASKREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StealTasks',
    full_name='ray.rpc.CoreWorkerService.StealTasks',
    index=1,
    containing_service=None,
    input_type=_STEALTASKSREQUEST,
    output_type=_STEALTASKSREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DirectActorCallArgWaitComplete',
    full_name='ray.rpc.CoreWorkerService.DirectActorCallArgWaitComplete',
    index=2,
    containing_service=None,
    input_type=_DIRECTACTORCALLARGWAITCOMPLETEREQUEST,
    output_type=_DIRECTACTORCALLARGWAITCOMPLETEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetObjectStatus',
    full_name='ray.rpc.CoreWorkerService.GetObjectStatus',
    index=3,
    containing_service=None,
    input_type=_GETOBJECTSTATUSREQUEST,
    output_type=_GETOBJECTSTATUSREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='WaitForActorOutOfScope',
    full_name='ray.rpc.CoreWorkerService.WaitForActorOutOfScope',
    index=4,
    containing_service=None,
    input_type=_WAITFORACTOROUTOFSCOPEREQUEST,
    output_type=_WAITFORACTOROUTOFSCOPEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PubsubLongPolling',
    full_name='ray.rpc.CoreWorkerService.PubsubLongPolling',
    index=5,
    containing_service=None,
    input_type=src_dot_ray_dot_protobuf_dot_pubsub__pb2._PUBSUBLONGPOLLINGREQUEST,
    output_type=src_dot_ray_dot_protobuf_dot_pubsub__pb2._PUBSUBLONGPOLLINGREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PubsubCommandBatch',
    full_name='ray.rpc.CoreWorkerService.PubsubCommandBatch',
    index=6,
    containing_service=None,
    input_type=src_dot_ray_dot_protobuf_dot_pubsub__pb2._PUBSUBCOMMANDBATCHREQUEST,
    output_type=src_dot_ray_dot_protobuf_dot_pubsub__pb2._PUBSUBCOMMANDBATCHREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AddObjectLocationOwner',
    full_name='ray.rpc.CoreWorkerService.AddObjectLocationOwner',
    index=7,
    containing_service=None,
    input_type=_ADDOBJECTLOCATIONOWNERREQUEST,
    output_type=_ADDOBJECTLOCATIONOWNERREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveObjectLocationOwner',
    full_name='ray.rpc.CoreWorkerService.RemoveObjectLocationOwner',
    index=8,
    containing_service=None,
    input_type=_REMOVEOBJECTLOCATIONOWNERREQUEST,
    output_type=_REMOVEOBJECTLOCATIONOWNERREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetObjectLocationsOwner',
    full_name='ray.rpc.CoreWorkerService.GetObjectLocationsOwner',
    index=9,
    containing_service=None,
    input_type=_GETOBJECTLOCATIONSOWNERREQUEST,
    output_type=_GETOBJECTLOCATIONSOWNERREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='KillActor',
    full_name='ray.rpc.CoreWorkerService.KillActor',
    index=10,
    containing_service=None,
    input_type=_KILLACTORREQUEST,
    output_type=_KILLACTORREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CancelTask',
    full_name='ray.rpc.CoreWorkerService.CancelTask',
    index=11,
    containing_service=None,
    input_type=_CANCELTASKREQUEST,
    output_type=_CANCELTASKREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoteCancelTask',
    full_name='ray.rpc.CoreWorkerService.RemoteCancelTask',
    index=12,
    containing_service=None,
    input_type=_REMOTECANCELTASKREQUEST,
    output_type=_REMOTECANCELTASKREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCoreWorkerStats',
    full_name='ray.rpc.CoreWorkerService.GetCoreWorkerStats',
    index=13,
    containing_service=None,
    input_type=_GETCOREWORKERSTATSREQUEST,
    output_type=_GETCOREWORKERSTATSREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='LocalGC',
    full_name='ray.rpc.CoreWorkerService.LocalGC',
    index=14,
    containing_service=None,
    input_type=_LOCALGCREQUEST,
    output_type=_LOCALGCREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SpillObjects',
    full_name='ray.rpc.CoreWorkerService.SpillObjects',
    index=15,
    containing_service=None,
    input_type=_SPILLOBJECTSREQUEST,
    output_type=_SPILLOBJECTSREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RestoreSpilledObjects',
    full_name='ray.rpc.CoreWorkerService.RestoreSpilledObjects',
    index=16,
    containing_service=None,
    input_type=_RESTORESPILLEDOBJECTSREQUEST,
    output_type=_RESTORESPILLEDOBJECTSREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteSpilledObjects',
    full_name='ray.rpc.CoreWorkerService.DeleteSpilledObjects',
    index=17,
    containing_service=None,
    input_type=_DELETESPILLEDOBJECTSREQUEST,
    output_type=_DELETESPILLEDOBJECTSREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AddSpilledUrl',
    full_name='ray.rpc.CoreWorkerService.AddSpilledUrl',
    index=18,
    containing_service=None,
    input_type=_ADDSPILLEDURLREQUEST,
    output_type=_ADDSPILLEDURLREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PlasmaObjectReady',
    full_name='ray.rpc.CoreWorkerService.PlasmaObjectReady',
    index=19,
    containing_service=None,
    input_type=_PLASMAOBJECTREADYREQUEST,
    output_type=_PLASMAOBJECTREADYREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RunOnUtilWorker',
    full_name='ray.rpc.CoreWorkerService.RunOnUtilWorker',
    index=20,
    containing_service=None,
    input_type=_RUNONUTILWORKERREQUEST,
    output_type=_RUNONUTILWORKERREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Exit',
    full_name='ray.rpc.CoreWorkerService.Exit',
    index=21,
    containing_service=None,
    input_type=_EXITREQUEST,
    output_type=_EXITREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AssignObjectOwner',
    full_name='ray.rpc.CoreWorkerService.AssignObjectOwner',
    index=22,
    containing_service=None,
    input_type=_ASSIGNOBJECTOWNERREQUEST,
    output_type=_ASSIGNOBJECTOWNERREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_COREWORKERSERVICE)

DESCRIPTOR.services_by_name['CoreWorkerService'] = _COREWORKERSERVICE

# @@protoc_insertion_point(module_scope)



import gym

from ray.rllib.env.vector_env import VectorEnv
from ray.rllib.utils.annotations import override


class MockEnv(gym.Env):
    """Mock environment for testing purposes.

    Observation=0, reward=1.0, episode-len is configurable.
    Actions are ignored.
    """

    def __init__(self, episode_length, config=None):
        self.episode_length = episode_length
        self.config = config
        self.i = 0
        self.observation_space = gym.spaces.Discrete(1)
        self.action_space = gym.spaces.Discrete(2)

    def reset(self):
        self.i = 0
        return 0

    def step(self, action):
        self.i += 1
        return 0, 1.0, self.i >= self.episode_length, {}


class MockEnv2(gym.Env):
    """Mock environment for testing purposes.

    Observation=ts (discrete space!), reward=100.0, episode-len is
    configurable. Actions are ignored.
    """

    def __init__(self, episode_length):
        self.episode_length = episode_length
        self.i = 0
        self.observation_space = gym.spaces.Discrete(100)
        self.action_space = gym.spaces.Discrete(2)

    def reset(self):
        self.i = 0
        return self.i

    def step(self, action):
        self.i += 1
        return self.i, 100.0, self.i >= self.episode_length, {}


class VectorizedMockEnv(VectorEnv):
    """Vectorized version of the MockEnv.

    Contains `num_envs` MockEnv instances, each one having its own
    `episode_length` horizon.
    """

    def __init__(self, episode_length, num_envs):
        super().__init__(
            observation_space=gym.spaces.Discrete(1),
            action_space=gym.spaces.Discrete(2),
            num_envs=num_envs)
        self.envs = [MockEnv(episode_length) for _ in range(num_envs)]

    @override(VectorEnv)
    def vector_reset(self):
        return [e.reset() for e in self.envs]

    @override(VectorEnv)
    def reset_at(self, index):
        return self.envs[index].reset()

    @override(VectorEnv)
    def vector_step(self, actions):
        obs_batch, rew_batch, done_batch, info_batch = [], [], [], []
        for i in range(len(self.envs)):
            obs, rew, done, info = self.envs[i].step(actions[i])
            obs_batch.append(obs)
            rew_batch.append(rew)
            done_batch.append(done)
            info_batch.append(info)
        return obs_batch, rew_batch, done_batch, info_batch

    @override(VectorEnv)
    def get_unwrapped(self):
        return self.envs


class MockVectorEnv(VectorEnv):
    """A custom vector env that uses a single(!) CartPole sub-env.

    However, this env pretends to be a vectorized one to illustrate how one
    could create custom VectorEnvs w/o the need for actual vectorizations of
    sub-envs under the hood.
    """

    def __init__(self, episode_length, mocked_num_envs):
        self.env = gym.make("CartPole-v0")
        super().__init__(
            observation_space=self.env.observation_space,
            action_space=self.env.action_space,
            num_envs=mocked_num_envs)
        self.episode_len = episode_length
        self.ts = 0

    @override(VectorEnv)
    def vector_reset(self):
        obs = self.env.reset()
        return [obs for _ in range(self.num_envs)]

    @override(VectorEnv)
    def reset_at(self, index):
        self.ts = 0
        return self.env.reset()

    @override(VectorEnv)
    def vector_step(self, actions):
        self.ts += 1
        # Apply all actions sequentially to the same env.
        # Whether this would make a lot of sense is debatable.
        obs_batch, rew_batch, done_batch, info_batch = [], [], [], []
        for i in range(self.num_envs):
            obs, rew, done, info = self.env.step(actions[i])
            # Artificially terminate once time step limit has been reached.
            # Note: Also terminate, when underlying CartPole is terminated.
            if self.ts >= self.episode_len:
                done = True
            obs_batch.append(obs)
            rew_batch.append(rew)
            done_batch.append(done)
            info_batch.append(info)
            if done:
                remaining = self.num_envs - (i + 1)
                obs_batch.extend([obs for _ in range(remaining)])
                rew_batch.extend([rew for _ in range(remaining)])
                done_batch.extend([done for _ in range(remaining)])
                info_batch.extend([info for _ in range(remaining)])
                break
        return obs_batch, rew_batch, done_batch, info_batch

    @override(VectorEnv)
    def get_unwrapped(self):
        # You may also leave this method as-is, in which case, it would
        # return an empty list.
        return [self.env for _ in range(self.num_envs)]



"""
Decentralized Distributed PPO (DD-PPO)
======================================

Unlike APPO or PPO, learning is no longer done centralized in the trainer
process. Instead, gradients are computed remotely on each rollout worker and
all-reduced to sync them at each mini-batch. This allows each worker's GPU
to be used both for sampling and for training.

DD-PPO should be used if you have envs that require GPUs to function, or have
a very large model that cannot be effectively optimized with the GPUs available
on a single machine (DD-PPO allows scaling to arbitrary numbers of GPUs across
multiple nodes, unlike PPO/APPO which is limited to GPUs on a single node).

Paper reference: https://arxiv.org/abs/1911.00357
Note that unlike the paper, we currently do not implement straggler mitigation.
"""

import logging
import sys
import time

import ray
from ray.rllib.agents.ppo import ppo
from ray.rllib.evaluation.worker_set import WorkerSet
from ray.rllib.execution.rollout_ops import ParallelRollouts
from ray.rllib.execution.metric_ops import StandardMetricsReporting
from ray.rllib.execution.common import STEPS_SAMPLED_COUNTER, \
    STEPS_TRAINED_COUNTER, LEARNER_INFO, LEARN_ON_BATCH_TIMER, \
    _get_shared_metrics, _get_global_vars
from ray.rllib.evaluation.rollout_worker import get_global_worker
from ray.rllib.utils.sgd import do_minibatch_sgd
from ray.rllib.utils.typing import TrainerConfigDict
from ray.util.iter import LocalIterator

logger = logging.getLogger(__name__)

# yapf: disable
# __sphinx_doc_begin__

# Adds the following updates to the `PPOTrainer` config in
# rllib/agents/ppo/ppo.py.
DEFAULT_CONFIG = ppo.PPOTrainer.merge_trainer_configs(
    ppo.DEFAULT_CONFIG,
    {
        # During the sampling phase, each rollout worker will collect a batch
        # `rollout_fragment_length * num_envs_per_worker` steps in size.
        "rollout_fragment_length": 100,
        # Vectorize the env (should enable by default since each worker has
        # a GPU).
        "num_envs_per_worker": 5,
        # During the SGD phase, workers iterate over minibatches of this size.
        # The effective minibatch size will be:
        # `sgd_minibatch_size * num_workers`.
        "sgd_minibatch_size": 50,
        # Number of SGD epochs per optimization round.
        "num_sgd_iter": 10,
        # Download weights between each training step. This adds a bit of
        # overhead but allows the user to access the weights from the trainer.
        "keep_local_weights_in_sync": True,

        # *** WARNING: configs below are DDPPO overrides over PPO; you
        #     shouldn't need to adjust them. ***
        # DDPPO requires PyTorch distributed.
        "framework": "torch",
        # The communication backend for PyTorch distributed.
        "torch_distributed_backend": "gloo",
        # Learning is no longer done on the driver process, so
        # giving GPUs to the driver does not make sense!
        "num_gpus": 0,
        # Each rollout worker gets a GPU.
        "num_gpus_per_worker": 1,
        # Require evenly sized batches. Otherwise,
        # collective allreduce could fail.
        "truncate_episodes": True,
        # This is auto set based on sample batch size.
        "train_batch_size": -1,
    },
    _allow_unknown_configs=True,
)

# __sphinx_doc_end__
# yapf: enable


def validate_config(config):
    """Validates the Trainer's config dict.

    Args:
        config (TrainerConfigDict): The Trainer's config to check.

    Raises:
        ValueError: In case something is wrong with the config.
    """

    # Error if run on Win.
    if sys.platform in ["win32", "cygwin"]:
        raise ValueError("DD-PPO not supported on Win yet! "
                         "Due to usage of torch.distributed.")

    # Auto-train_batch_size: Calculate from rollout len and envs-per-worker.
    if config["train_batch_size"] == -1:
        config["train_batch_size"] = (
            config["rollout_fragment_length"] * config["num_envs_per_worker"])
    # Users should not define `train_batch_size` directly (always -1).
    else:
        raise ValueError(
            "Set rollout_fragment_length instead of train_batch_size "
            "for DDPPO.")

    # Only supported for PyTorch so far.
    if config["framework"] != "torch":
        raise ValueError(
            "Distributed data parallel is only supported for PyTorch")
    if config["torch_distributed_backend"] not in ("gloo", "mpi", "nccl"):
        raise ValueError("Only gloo, mpi, or nccl is supported for "
                         "the backend of PyTorch distributed.")
    # `num_gpus` must be 0/None, since all optimization happens on Workers.
    if config["num_gpus"]:
        raise ValueError(
            "When using distributed data parallel, you should set "
            "num_gpus=0 since all optimization "
            "is happening on workers. Enable GPUs for workers by setting "
            "num_gpus_per_worker=1.")
    # `batch_mode` must be "truncate_episodes".
    if config["batch_mode"] != "truncate_episodes":
        raise ValueError(
            "Distributed data parallel requires truncate_episodes "
            "batch mode.")
    # Call (base) PPO's config validation function.
    ppo.validate_config(config)


def execution_plan(workers: WorkerSet,
                   config: TrainerConfigDict) -> LocalIterator[dict]:
    """Execution plan of the DD-PPO algorithm. Defines the distributed dataflow.

    Args:
        workers (WorkerSet): The WorkerSet for training the Polic(y/ies)
            of the Trainer.
        config (TrainerConfigDict): The trainer's configuration dict.

    Returns:
        LocalIterator[dict]: The Policy class to use with PGTrainer.
            If None, use `default_policy` provided in build_trainer().
    """
    rollouts = ParallelRollouts(workers, mode="raw")

    # Setup the distributed processes.
    if not workers.remote_workers():
        raise ValueError("This optimizer requires >0 remote workers.")
    ip = ray.get(workers.remote_workers()[0].get_node_ip.remote())
    port = ray.get(workers.remote_workers()[0].find_free_port.remote())
    address = "tcp://{ip}:{port}".format(ip=ip, port=port)
    logger.info("Creating torch process group with leader {}".format(address))

    # Get setup tasks in order to throw errors on failure.
    ray.get([
        worker.setup_torch_data_parallel.remote(
            address,
            i,
            len(workers.remote_workers()),
            backend=config["torch_distributed_backend"])
        for i, worker in enumerate(workers.remote_workers())
    ])
    logger.info("Torch process group init completed")

    # This function is applied remotely on each rollout worker.
    def train_torch_distributed_allreduce(batch):
        expected_batch_size = (
            config["rollout_fragment_length"] * config["num_envs_per_worker"])
        this_worker = get_global_worker()
        assert batch.count == expected_batch_size, \
            ("Batch size possibly out of sync between workers, expected:",
             expected_batch_size, "got:", batch.count)
        logger.info("Executing distributed minibatch SGD "
                    "with epoch size {}, minibatch size {}".format(
                        batch.count, config["sgd_minibatch_size"]))
        info = do_minibatch_sgd(batch, this_worker.policy_map, this_worker,
                                config["num_sgd_iter"],
                                config["sgd_minibatch_size"], ["advantages"])
        return info, batch.count

    # Broadcast the local set of global vars.
    def update_worker_global_vars(item):
        global_vars = _get_global_vars()
        for w in workers.remote_workers():
            w.set_global_vars.remote(global_vars)
        return item

    # Have to manually record stats since we are using "raw" rollouts mode.
    class RecordStats:
        def _on_fetch_start(self):
            self.fetch_start_time = time.perf_counter()

        def __call__(self, items):
            for item in items:
                info, count = item
                metrics = _get_shared_metrics()
                metrics.counters[STEPS_SAMPLED_COUNTER] += count
                metrics.counters[STEPS_TRAINED_COUNTER] += count
                metrics.info[LEARNER_INFO] = info
            # Since SGD happens remotely, the time delay between fetch and
            # completion is approximately the SGD step time.
            metrics.timers[LEARN_ON_BATCH_TIMER].push(time.perf_counter() -
                                                      self.fetch_start_time)

    train_op = (
        rollouts.for_each(train_torch_distributed_allreduce)  # allreduce
        .batch_across_shards()  # List[(grad_info, count)]
        .for_each(RecordStats()))

    train_op = train_op.for_each(update_worker_global_vars)

    # Sync down the weights. As with the sync up, this is not really
    # needed unless the user is reading the local weights.
    if config["keep_local_weights_in_sync"]:

        def download_weights(item):
            workers.local_worker().set_weights(
                ray.get(workers.remote_workers()[0].get_weights.remote()))
            return item

        train_op = train_op.for_each(download_weights)

    # In debug mode, check the allreduce successfully synced the weights.
    if logger.isEnabledFor(logging.DEBUG):

        def check_sync(item):
            weights = ray.get(
                [w.get_weights.remote() for w in workers.remote_workers()])
            sums = []
            for w in weights:
                acc = 0
                for p in w.values():
                    for k, v in p.items():
                        acc += v.sum()
                sums.append(float(acc))
            logger.debug("The worker weight sums are {}".format(sums))
            assert len(set(sums)) == 1, sums

        train_op = train_op.for_each(check_sync)

    return StandardMetricsReporting(train_op, workers, config)


# Build a child class of `Trainer`, based on PPOTrainer's setup.
# Note: The generated class is NOT a sub-class of PPOTrainer, but directly of
# the `Trainer` class.
DDPPOTrainer = ppo.PPOTrainer.with_updates(
    name="DDPPO",
    default_config=DEFAULT_CONFIG,
    validate_config=validate_config,
    execution_plan=execution_plan,
)



"""PyTorch model for DQN"""

from typing import Sequence
import gym
from ray.rllib.models.torch.misc import SlimFC
from ray.rllib.models.torch.modules.noisy_layer import NoisyLayer
from ray.rllib.models.torch.torch_modelv2 import TorchModelV2
from ray.rllib.utils.framework import try_import_torch
from ray.rllib.utils.typing import ModelConfigDict

torch, nn = try_import_torch()


class DQNTorchModel(TorchModelV2, nn.Module):
    """Extension of standard TorchModelV2 to provide dueling-Q functionality.
    """

    def __init__(
            self,
            obs_space: gym.spaces.Space,
            action_space: gym.spaces.Space,
            num_outputs: int,
            model_config: ModelConfigDict,
            name: str,
            *,
            q_hiddens: Sequence[int] = (256, ),
            dueling: bool = False,
            dueling_activation: str = "relu",
            num_atoms: int = 1,
            use_noisy: bool = False,
            v_min: float = -10.0,
            v_max: float = 10.0,
            sigma0: float = 0.5,
            # TODO(sven): Move `add_layer_norm` into ModelCatalog as
            #  generic option, then error if we use ParameterNoise as
            #  Exploration type and do not have any LayerNorm layers in
            #  the net.
            add_layer_norm: bool = False):
        """Initialize variables of this model.

        Extra model kwargs:
            q_hiddens (Sequence[int]): List of layer-sizes after(!) the
                Advantages(A)/Value(V)-split. Hence, each of the A- and V-
                branches will have this structure of Dense layers. To define
                the NN before this A/V-split, use - as always -
                config["model"]["fcnet_hiddens"].
            dueling (bool): Whether to build the advantage(A)/value(V) heads
                for DDQN. If True, Q-values are calculated as:
                Q = (A - mean[A]) + V. If False, raw NN output is interpreted
                as Q-values.
            dueling_activation (str): The activation to use for all dueling
                layers (A- and V-branch). One of "relu", "tanh", "linear".
            num_atoms (int): If >1, enables distributional DQN.
            use_noisy (bool): Use noisy layers.
            v_min (float): Min value support for distributional DQN.
            v_max (float): Max value support for distributional DQN.
            sigma0 (float): Initial value of noisy layers.
            add_layer_norm (bool): Enable layer norm (for param noise).
        """
        nn.Module.__init__(self)
        super(DQNTorchModel, self).__init__(obs_space, action_space,
                                            num_outputs, model_config, name)

        self.dueling = dueling
        self.num_atoms = num_atoms
        self.v_min = v_min
        self.v_max = v_max
        self.sigma0 = sigma0
        ins = num_outputs

        advantage_module = nn.Sequential()
        value_module = nn.Sequential()

        # Dueling case: Build the shared (advantages and value) fc-network.
        for i, n in enumerate(q_hiddens):
            if use_noisy:
                advantage_module.add_module(
                    "dueling_A_{}".format(i),
                    NoisyLayer(
                        ins,
                        n,
                        sigma0=self.sigma0,
                        activation=dueling_activation))
                value_module.add_module(
                    "dueling_V_{}".format(i),
                    NoisyLayer(
                        ins,
                        n,
                        sigma0=self.sigma0,
                        activation=dueling_activation))
            else:
                advantage_module.add_module(
                    "dueling_A_{}".format(i),
                    SlimFC(ins, n, activation_fn=dueling_activation))
                value_module.add_module(
                    "dueling_V_{}".format(i),
                    SlimFC(ins, n, activation_fn=dueling_activation))
                # Add LayerNorm after each Dense.
                if add_layer_norm:
                    advantage_module.add_module("LayerNorm_A_{}".format(i),
                                                nn.LayerNorm(n))
                    value_module.add_module("LayerNorm_V_{}".format(i),
                                            nn.LayerNorm(n))
            ins = n

        # Actual Advantages layer (nodes=num-actions).
        if use_noisy:
            advantage_module.add_module(
                "A",
                NoisyLayer(
                    ins,
                    self.action_space.n * self.num_atoms,
                    sigma0,
                    activation=None))
        elif q_hiddens:
            advantage_module.add_module(
                "A",
                SlimFC(
                    ins, action_space.n * self.num_atoms, activation_fn=None))

        self.advantage_module = advantage_module

        # Value layer (nodes=1).
        if self.dueling:
            if use_noisy:
                value_module.add_module(
                    "V",
                    NoisyLayer(ins, self.num_atoms, sigma0, activation=None))
            elif q_hiddens:
                value_module.add_module(
                    "V", SlimFC(ins, self.num_atoms, activation_fn=None))
            self.value_module = value_module

    def get_q_value_distributions(self, model_out):
        """Returns distributional values for Q(s, a) given a state embedding.

        Override this in your custom model to customize the Q output head.

        Args:
            model_out (Tensor): Embedding from the model layers.

        Returns:
            (action_scores, logits, dist) if num_atoms == 1, otherwise
            (action_scores, z, support_logits_per_action, logits, dist)
        """
        action_scores = self.advantage_module(model_out)

        if self.num_atoms > 1:
            # Distributional Q-learning uses a discrete support z
            # to represent the action value distribution
            z = torch.range(
                0.0, self.num_atoms - 1,
                dtype=torch.float32).to(action_scores.device)
            z = self.v_min + \
                z * (self.v_max - self.v_min) / float(self.num_atoms - 1)

            support_logits_per_action = torch.reshape(
                action_scores, shape=(-1, self.action_space.n, self.num_atoms))
            support_prob_per_action = nn.functional.softmax(
                support_logits_per_action, dim=-1)
            action_scores = torch.sum(z * support_prob_per_action, dim=-1)
            logits = support_logits_per_action
            probs = support_prob_per_action
            return action_scores, z, support_logits_per_action, logits, probs
        else:
            logits = torch.unsqueeze(torch.ones_like(action_scores), -1)
            return action_scores, logits, logits

    def get_state_value(self, model_out):
        """Returns the state value prediction for the given state embedding."""

        return self.value_module(model_out)



import gym

from ray.rllib.models.modelv2 import ModelV2
from ray.rllib.utils.annotations import PublicAPI
from ray.rllib.utils.typing import ModelConfigDict


@PublicAPI
class JAXModelV2(ModelV2):
    """JAX version of ModelV2.

    Note that this class by itself is not a valid model unless you
    implement forward() in a subclass."""

    def __init__(self, obs_space: gym.spaces.Space,
                 action_space: gym.spaces.Space, num_outputs: int,
                 model_config: ModelConfigDict, name: str):
        """Initializes a JAXModelV2 instance."""

        ModelV2.__init__(
            self,
            obs_space,
            action_space,
            num_outputs,
            model_config,
            name,
            framework="jax")



from ray.serve.api import (connect, start, get_replica_context, get_handle,
                           shadow_traffic, set_traffic, delete_backend,
                           list_backends, create_backend, get_backend_config,
                           update_backend_config, list_endpoints,
                           delete_endpoint, create_endpoint, shutdown, ingress,
                           deployment, get_deployment, list_deployments)
from ray.serve.batching import batch
from ray.serve.config import BackendConfig, HTTPOptions
from ray.serve.utils import ServeRequest

# Mute the warning because Serve sometimes intentionally calls
# ray.get inside async actors.
import ray.worker
ray.worker.blocking_get_inside_async_warned = True

__all__ = [
    "BackendConfig", "batch", "connect", "start", "HTTPOptions",
    "get_replica_context", "ServeRequest", "get_handle", "shadow_traffic",
    "set_traffic", "delete_backend", "list_backends", "create_backend",
    "get_backend_config", "update_backend_config", "list_endpoints",
    "delete_endpoint", "create_endpoint", "shutdown", "ingress", "deployment",
    "get_deployment", "list_deployments"
]



import gym
import numpy as np
import random
from ray.rllib.policy.policy import Policy
from ray.rllib.policy.view_requirement import ViewRequirement

ROCK = 0
PAPER = 1
SCISSORS = 2


class AlwaysSameHeuristic(Policy):
    """Pick a random move and stick with it for the entire episode."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.exploration = self._create_exploration()
        self.view_requirements.update({
            "state_in_0": ViewRequirement(
                "state_out_0",
                shift=-1,
                space=gym.spaces.Box(0, 100, shape=(), dtype=np.int32))
        })

    def get_initial_state(self):
        return [random.choice([ROCK, PAPER, SCISSORS])]

    def compute_actions(self,
                        obs_batch,
                        state_batches=None,
                        prev_action_batch=None,
                        prev_reward_batch=None,
                        info_batch=None,
                        episodes=None,
                        **kwargs):
        return state_batches[0], state_batches, {}


class BeatLastHeuristic(Policy):
    """Play the move that would beat the last move of the opponent."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.exploration = self._create_exploration()

    def compute_actions(self,
                        obs_batch,
                        state_batches=None,
                        prev_action_batch=None,
                        prev_reward_batch=None,
                        info_batch=None,
                        episodes=None,
                        **kwargs):
        def successor(x):
            if x[ROCK] == 1:
                return PAPER
            elif x[PAPER] == 1:
                return SCISSORS
            elif x[SCISSORS] == 1:
                return ROCK
            elif x[-1] == 1:
                return random.choice([ROCK, PAPER, SCISSORS])

        return [successor(x) for x in obs_batch], [], {}

    def learn_on_batch(self, samples):
        pass

    def get_weights(self):
        pass

    def set_weights(self, weights):
        pass



import logging

from ray.rllib.agents.trainer import with_common_config
from ray.rllib.agents.trainer_template import build_trainer
from ray.rllib.contrib.bandits.agents.policy import BanditPolicy

logger = logging.getLogger(__name__)

# yapf: disable
# __sphinx_doc_begin__
TS_CONFIG = with_common_config({
    # No remote workers by default.
    "num_workers": 0,
    "framework": "torch",  # Only PyTorch supported so far.

    # Do online learning one step at a time.
    "rollout_fragment_length": 1,
    "train_batch_size": 1,

    # Bandits cant afford to do one timestep per iteration as it is extremely
    # slow because of metrics collection overhead. This setting means that the
    # agent will be trained for 100 times in one iteration of Rllib
    "timesteps_per_iteration": 100,

    "exploration_config": {
        "type": "ray.rllib.contrib.bandits.exploration.ThompsonSampling"
    }
})
# __sphinx_doc_end__
# yapf: enable

LinTSTrainer = build_trainer(
    name="LinTS", default_config=TS_CONFIG, default_policy=BanditPolicy)
